@@ -1,4 +1,4 @@
-# $Id: PerlSignals.pm 1903 2006-03-20 04:44:08Z rcaputo $
+# $Id: PerlSignals.pm 2447 2009-02-17 05:04:43Z rcaputo $
 
 # Plain Perl signal handling is something shared by several event
 # loops.  The invariant code has moved out here so that each loop may
@@ -10,7 +10,7 @@ package POE::Loop::PerlSignals;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 1903 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 2447 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 # Everything plugs into POE::Kernel.
 package POE::Kernel;
@@ -18,6 +18,10 @@ package POE::Kernel;
 use strict;
 use POE::Kernel;
 
+# Flag so we know which signals are watched.  Used to reset those
+# signals during finalization.
+my %signal_watched;
+
 #------------------------------------------------------------------------------
 # Signal handlers/callbacks.
 
@@ -45,18 +49,40 @@ sub _loop_signal_handler_pipe {
   $SIG{$_[0]} = \&_loop_signal_handler_pipe;
 }
 
+# only used under USE_SIGCHLD
+sub _loop_signal_handler_chld {
+  if (TRACE_SIGNALS) {
+    POE::Kernel::_warn "<sg> Enqueuing CHLD-like SIG$_[0] event";
+  }
+
+  $poe_kernel->_data_sig_enqueue_poll_event();
+  $SIG{$_[0]} = \&_loop_signal_handler_chld;
+}
+
 #------------------------------------------------------------------------------
 # Signal handler maintenance functions.
 
 sub loop_watch_signal {
   my ($self, $signal) = @_;
 
+  $signal_watched{$signal} = 1;
+
   # Child process has stopped.
   if ($signal eq 'CHLD' or $signal eq 'CLD') {
-# We should never twiddle $SIG{CH?LD} under poe, unless we want to override
-# system() and friends. --hachi
-#    $SIG{$signal} = "DEFAULT";
-    $self->_data_sig_begin_polling();
+    if ( USE_SIGCHLD ) {
+      # install, but also trigger once
+      # there may be a race condition between forking, and $kernel->sig_chld in
+      # which the signal is already delivered
+      # and the interval polling mechanism will still generate a SIGCHLD
+      # signal, this preserves that behavior
+      $SIG{$signal} = \&_loop_signal_handler_chld;
+      $self->_data_sig_enqueue_poll_event();
+    } else {
+      # We should never twiddle $SIG{CH?LD} under POE, unless we want to
+      # override system() and friends. --hachi
+      # $SIG{$signal} = "DEFAULT";
+      $self->_data_sig_begin_polling();
+    }
     return;
   }
 
@@ -73,12 +99,16 @@ sub loop_watch_signal {
 sub loop_ignore_signal {
   my ($self, $signal) = @_;
 
-  if ($signal eq 'CHLD' or $signal eq 'CLD') {
-    $self->_data_sig_cease_polling();
-# We should never twiddle $SIG{CH?LD} under poe, unless we want to override
-# system() and friends. --hachi
-#    $SIG{$signal} = "IGNORE";
-    return;
+  delete $signal_watched{$signal};
+
+  unless ( USE_SIGCHLD ) {
+    if ($signal eq 'CHLD' or $signal eq 'CLD') {
+      $self->_data_sig_cease_polling();
+      # We should never twiddle $SIG{CH?LD} under poe, unless we want to
+      # override system() and friends. --hachi
+      # $SIG{$signal} = "IGNORE";
+      return;
+    }
   }
 
   if ($signal eq 'PIPE') {
@@ -89,13 +119,20 @@ sub loop_ignore_signal {
   $SIG{$signal} = "DEFAULT";
 }
 
+sub loop_ignore_all_signals {
+  my $self = shift;
+  foreach my $signal (keys %signal_watched) {
+    $self->loop_ignore_signal($signal);
+  }
+}
+
 1;
 
 __END__
 
 =head1 NAME
 
-POE::Loop::PerlSignals - plain Perl signal handlers used by many loops
+POE::Loop::PerlSignals - common signal handling routines for POE::Loop bridges
 
 =head1 SYNOPSIS
 
@@ -103,10 +140,12 @@ See L<POE::Loop>.
 
 =head1 DESCRIPTION
 
-This class is an implementation of the signal handling functions
-defined by the abstract POE::Loop interface.  It follows POE::Loop's
-public interface for signal handling exactly.  Therefore, please see
-L<POE::Loop> for its documentation.
+POE::Loop::PerlSignals implements common code to handle signals for
+many different event loops.  Most loops don't handle signals natively,
+so this code has been abstracted into a reusable mix-in module.
+
+POE::Loop::PerlSignals follows POE::Loop's public interface for signal
+handling.  Therefore, please see L<POE::Loop> for more details.
 
 =head1 SEE ALSO
 
@@ -118,3 +157,6 @@ Please see L<POE> for more information about authors, contributors,
 and POE's licensing.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
